'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _asyncToGenerator2;

function _load_asyncToGenerator() {
  return _asyncToGenerator2 = _interopRequireDefault(require('babel-runtime/helpers/asyncToGenerator'));
}

var _constants;

function _load_constants() {
  return _constants = _interopRequireWildcard(require('./constants.js'));
}

var _index;

function _load_index() {
  return _index = require('./registries/index.js');
}

var _wrapper;

function _load_wrapper() {
  return _wrapper = _interopRequireDefault(require('./lockfile/wrapper.js'));
}

var _crypto;

function _load_crypto() {
  return _crypto = _interopRequireWildcard(require('./util/crypto.js'));
}

var _fs;

function _load_fs() {
  return _fs = _interopRequireWildcard(require('./util/fs.js'));
}

var _misc;

function _load_misc() {
  return _misc = require('./util/misc.js');
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const invariant = require('invariant');
const path = require('path');

/**
 *
 */
class InstallationIntegrityChecker {
  constructor(config) {
    this.config = config;
  }

  /**
   * Get the location of an existing integrity hash. If none exists then return the location where we should
   * write a new one.
   */

  _getIntegrityHashLocation(usedRegistries) {
    var _this = this;

    return (0, (_asyncToGenerator2 || _load_asyncToGenerator()).default)(function* () {
      // build up possible folders
      let registries = (_index || _load_index()).registryNames;
      if (usedRegistries && usedRegistries.size > 0) {
        registries = usedRegistries;
      }
      const possibleFolders = [];
      if (_this.config.modulesFolder) {
        possibleFolders.push(_this.config.modulesFolder);
      }

      // ensure we only write to a registry folder that was used
      for (const name of registries) {
        const loc = path.join(_this.config.cwd, _this.config.registries[name].folder);
        possibleFolders.push(loc);
      }

      // if we already have an integrity hash in one of these folders then use it's location otherwise use the
      // first folder
      const possibles = possibleFolders.map(function (folder) {
        return path.join(folder, (_constants || _load_constants()).INTEGRITY_FILENAME);
      });
      let loc;
      for (const possibleLoc of possibles) {
        if (yield (_fs || _load_fs()).exists(possibleLoc)) {
          loc = possibleLoc;
          break;
        }
      }
      return {
        locationPath: loc || possibles[0],
        exists: !!loc
      };
    })();
  }

  /**
   * Generate integrity hash of input lockfile.
   */

  _generateIntegrityHash(lockfile, patterns, flags) {
    const opts = [lockfile];

    opts.push(`patterns:${patterns.sort((_misc || _load_misc()).sortAlpha).join(',')}`);

    if (flags.flat) {
      opts.push('flat');
    }

    if (this.config.production) {
      opts.push('production');
    }

    const linkedModules = this.config.linkedModules;
    if (linkedModules.length) {
      opts.push(`linked:${linkedModules.join(',')}`);
    }

    const mirror = this.config.getOfflineMirrorPath();
    if (mirror != null) {
      opts.push(`mirror:${mirror}`);
    }

    return (_crypto || _load_crypto()).hash(opts.join('-'), 'sha256');
  }

  check(patterns, lockfile, normalizedLockSource, flags) {
    var _this2 = this;

    return (0, (_asyncToGenerator2 || _load_asyncToGenerator()).default)(function* () {
      // check if patterns exist in lockfile
      const missingPatterns = patterns.filter(function (p) {
        return !lockfile.getLocked(p);
      });
      const loc = yield _this2._getIntegrityHashLocation();
      if (missingPatterns.length || !loc.exists) {
        return {
          integrityFileMissing: !loc.exists,
          missingPatterns
        };
      }

      const actual = _this2._generateIntegrityHash(normalizedLockSource, patterns, flags);
      const expected = (yield (_fs || _load_fs()).readFile(loc.locationPath)).trim();

      return {
        integrityFileMissing: false,
        integrityHashMatches: actual === expected,
        missingPatterns
      };
    })();
  }

  /**
   * Write the integrity hash of the current install to disk.
   */
  save(patterns, normalizedLockSource, flags, usedRegistries) {
    var _this3 = this;

    return (0, (_asyncToGenerator2 || _load_asyncToGenerator()).default)(function* () {
      const loc = yield _this3._getIntegrityHashLocation(usedRegistries);
      invariant(loc.locationPath, 'expected integrity hash location');
      yield (_fs || _load_fs()).mkdirp(path.dirname(loc.locationPath));
      yield (_fs || _load_fs()).writeFile(loc.locationPath, _this3._generateIntegrityHash(normalizedLockSource, patterns, flags));
    })();
  }

  removeIntegrityFile() {
    var _this4 = this;

    return (0, (_asyncToGenerator2 || _load_asyncToGenerator()).default)(function* () {
      const loc = yield _this4._getIntegrityHashLocation();
      if (loc.exists) {
        yield (_fs || _load_fs()).unlink(loc.locationPath);
      }
    })();
  }

}
exports.default = InstallationIntegrityChecker;